import { createRule } from "../createRule.js";
const defaultCollections = [
  "scripts",
  "devDependencies",
  "dependencies",
  "peerDependencies",
  "config"
];
const rule = createRule({
  create(context) {
    const toSort = context.options[0] || defaultCollections;
    return {
      "JSONProperty:exit"(node) {
        const { key, value } = node;
        const collection = value;
        if (collection.type === "JSONObjectExpression" && toSort.includes(key.value)) {
          const currentOrder = collection.properties;
          const desiredOrder = currentOrder.slice().sort(
            (a, b) => a.key.value > b.key.value ? 1 : -1
          );
          if (currentOrder.some(
            (property, i) => desiredOrder[i] !== property
          )) {
            context.report({
              data: {
                key: key.value
              },
              fix(fixer) {
                return fixer.replaceText(
                  collection,
                  JSON.stringify(
                    desiredOrder.reduce((out, property) => {
                      out[property.key.value] = JSON.parse(
                        context.sourceCode.getText(
                          property.value
                        )
                      );
                      return out;
                    }, {}),
                    null,
                    2
                  ).split("\n").join("\n  ")
                  // nest indents
                );
              },
              loc: collection.loc,
              message: "Package {{ key }} are not alphabetized",
              node
            });
          }
        }
      }
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Dependencies, scripts, and configuration values must be declared in alphabetical order.",
      recommended: true
    },
    fixable: "code",
    schema: [
      {
        items: {
          type: "string"
        },
        type: "array"
      }
    ]
  }
});
export {
  rule
};
//# sourceMappingURL=sort-collections.mjs.map