import { createRule } from "../createRule.js";
import { isJSONStringLiteral, isNotNullish } from "../utils/predicates.js";
const dependencyPropertyNames = /* @__PURE__ */ new Set([
  "bundleDependencies",
  "bundledDependencies",
  "dependencies",
  "devDependencies",
  "optionalDependencies",
  "peerDependencies",
  "overrides"
]);
const rule = createRule({
  create(context) {
    function check(elements, getNodeToRemove) {
      const seen = /* @__PURE__ */ new Set();
      for (const element of elements.filter(isNotNullish).filter(isJSONStringLiteral).reverse()) {
        if (seen.has(element.value)) {
          report(element);
        } else {
          seen.add(element.value);
        }
      }
      function report(node) {
        context.report({
          messageId: "overridden",
          node,
          suggest: [
            {
              fix(fixer) {
                const removal = getNodeToRemove(node);
                return [
                  fixer.remove(removal),
                  fixer.remove(
                    // A listing that's overridden can't be last,
                    // so we're guaranteed there's a comma after.
                    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                    context.sourceCode.getTokenAfter(
                      removal
                    )
                  )
                ];
              },
              messageId: "remove"
            }
          ]
        });
      }
    }
    return {
      "Program > JSONExpressionStatement > JSONObjectExpression > JSONProperty[key.type=JSONLiteral]"(node) {
        if (!dependencyPropertyNames.has(node.key.value)) {
          return;
        }
        switch (node.value.type) {
          case "JSONArrayExpression":
            check(
              node.value.elements,
              (element) => element
            );
            break;
          case "JSONObjectExpression":
            check(
              node.value.properties.map(
                (property) => property.key
              ),
              (property) => property.parent
            );
            break;
        }
      }
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Enforce that if repository directory is specified, it matches the path to the package.json file",
      recommended: true
    },
    hasSuggestions: true,
    messages: {
      overridden: "Package name is overridden by a duplicate listing later on.",
      remove: "Remove this redundant dependency listing."
    }
  }
});
export {
  rule
};
//# sourceMappingURL=unique-dependencies.mjs.map